@@ -1,4 +1,4 @@
-/* $Id: files.c 4299 2008-08-09 09:31:33Z astyanax $ */
+/* $Id: files.c 4317 2008-08-30 21:00:00Z astyanax $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
@@ -1588,7 +1588,7 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
 	}
 
 	if (f_open == NULL) {
-	    fd_source = open(realname, O_RDONLY | O_CREAT);
+	    fd_source = open(realname, O_RDONLY | O_CREAT, S_IRUSR | S_IWUSR);
 
 	    if (fd_source != -1) {
 		f_source = fdopen(fd_source, "rb");
@@ -1707,7 +1707,7 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
 	int fd_source;
 	FILE *f_source = NULL;
 
-	fd_source = open(tempname, O_RDONLY | O_CREAT);
+	fd_source = open(tempname, O_RDONLY | O_CREAT, S_IRUSR | S_IWUSR);
 
 	if (fd_source != -1) {
 	    f_source = fdopen(fd_source, "rb");
@@ -1728,8 +1728,11 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
 		strerror(errno));
 	    goto cleanup_and_exit;
 	}
-    } else
-	fclose(f);
+    } else if (fclose(f) != 0) {
+	    statusbar(_("Error writing %s: %s"), realname,
+		strerror(errno));
+	    goto cleanup_and_exit;
+    }
 
     if (!tmp && append == OVERWRITE) {
 	if (!nonamechange) {
@@ -1945,6 +1948,8 @@ bool do_writeout(bool exiting)
 	    } else if (s && s->scfunc == (void *) append_msg) {
 		append = (append == APPEND) ? OVERWRITE : APPEND;
 		continue;
+	    } else if (s && s->scfunc == do_help_void) {
+		continue;
 	    }
 
 #ifdef DEBUG
