@@ -1,4 +1,4 @@
-/* $Id: nano.h 4291 2008-08-03 04:48:05Z astyanax $ */
+/* $Id: nano.h 4328 2008-09-21 23:02:30Z astyanax $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
@@ -242,6 +242,8 @@ typedef struct syntaxtype {
 	/* The name of this syntax. */
     exttype *extensions;
 	/* The list of extensions that this syntax applies to. */
+    exttype *headers;
+	/* Regexes to match on the 'header' (1st line) of the file */
     colortype *color;
 	/* The colors used in this syntax. */
     struct syntaxtype *next;
